package My::File;
use Moose;
use File::Basename;
use Image::Size;

=encoding utf8

=head1 NAME

My::File - a class for representing a plain file.  Depends on B<Moose>,
B<File::Basename> and B<Image::Size>.

=head1 SYNOPSIS

  use My::File;
  my $f = My::File->new(path => "path/to/aardvark.jpg");
  printf("I found this file in %s\n", $f->dir);

=head1 METHODS

The examples below use the same path as in the synopsis above.

=over

=item B<new>

The constructor, which requires exactly one key-value pair for the
file path, as in the synopsis.

=item B<path>

The path to the file. It must be an existing plain file.

The directory may be omitted for a file in the current working
directory. The directory may be absolute or relative to the current
working directory.

E.g. 'path/to/aardvark.jpg'.

=item B<dir>

E.g. 'path/to' (it has no trailing '/').

=item B<ext>

E.g. 'jpg' (it has no leading '.').

=item B<name>

E.g. 'aardvark.jpg'.

=item B<nm>

E.g. 'aardvark'.

=item B<height>

If the file is a .jpg, .png or .gif then this is the image height in
pixels, as provided by Image::Size.  E.g. 200.  For other files it
returns undef.

=item B<width>

Same as height, except it's width. E.g. 300 or undef.

=back

=head1 COPYING

©Copyright <webmaster@acrasis.net>. This is free software; you may
redistribute it and/or modify it under the same terms as Perl itself.

=cut

has path => ( is  => 'ro', required => 1, isa => 'Str' );
has dir => ( is  => 'ro', isa => 'Str', writer => '_dir' );
has ext => ( is  => 'ro', isa => 'Str', writer => '_ext' );
has name => ( is  => 'ro', isa => 'Str', writer => '_name' );
has nm => ( is  => 'ro', isa => 'Str', writer => '_nm' );
has height => ( is => 'ro', isa => 'Int', writer => '_height' );
has width => ( is => 'ro', isa => 'Int', writer => '_width' );

our $EXT_REGEX = qr/\.[^.]*$/;

sub BUILD {
   my ($self) = @_;
   die(sprintf("%s: no such plain file\n", $self->path)) if (! -f $self->path);
   my ($nm, $dir, $ext) = fileparse($self->path, $EXT_REGEX);

   # Prefer dir to omit its trailing '/'.
   $dir =~ s[/$][];
   # Prefer ext to omit its leading '.'.
   $ext =~ s/^\.//;

   $self->_dir($dir);
   $self->_nm($nm);
   $self->_ext($ext);
   $self->_name("$nm.$ext");

   if ($ext =~ m/^(jpe?g|png|gif)$/i) {
      my ($w, $h) = imgsize($self->path);
      $self->_width($w);
      $self->_height($h);
   }

   return;
}

no Moose;
__PACKAGE__->meta->make_immutable;
