package My::Glob;
use Moose;
use My::File;
extends 'My::Plugin';

=encoding utf8

=head1 NAME

My::Glob - a Template Toolkit plugin for getting a list of files that
match a shell glob pattern.

Depends also on B<My::Plugin> and B<My::File>.

=head1 SYNOPSIS

You'd typically use this in a template, for example to show all the
thumbnail images in a directory:

   [% USE gfiles = Glob('*-thumb.jpg') -%]
   [% FOREACH file = gfiles.files -%]
      <img src="[% file.name %]"
           width="[% file.width %]" height="[% file.height %]">
   [% END -%]

=head1 CONSTRUCTION

The line in the Synopsis

   [% USE gfiles = Glob('*-thumb.jpg') -%]

shows the My::Glob object named C<gfiles> constructed with the pattern
C<*-thumb.jpg>.  The pattern is treated as relative to the current
template file's directory.

So if you have a template C<pics/cats/index.tt2> with the markup shown
in the Synopsis, it will show the images
C<pics/cats/01-fatcat-thumb.jpg>, C<pics/cats/02-kitten-thumb.jpg> and
so forth.

=head1 METHODS

=over

=item B<files>

Returns the files that matched the glob pattern, as a Template Toolkit
iterator.  Each file is represented by a B<My::File> object.

=back

=head1 COPYING

©Copyright <webmaster@acrasis.net>. This is free software; you may
redistribute it and/or modify it under the same terms as Perl itself.

=cut

our $VERSION = 0.1;

has pat => (is => 'ro', required => 1, isa => 'Str');

has files => (
   is => 'ro', required => 0, isa => 'ArrayRef[My::File]',
   builder => '_build_files', lazy => 1,
);

# This is ugly but there is apparently no standard pattern for using
# BUILDARGS in an inheritance hierarchy.  The doctrine I'm following
# here is:
#
# The concrete class removes (and remembers) those elements from @_
# that it know it needs.  It passes what's left to its superclass's
# BUILDARGS, receiving a hashref.
#
# The concrete class takes that hashref and adds the key-val pairs
# that it knows about, using what it removed earlier.
#
sub BUILDARGS {
   # Expecting @_ to contain (class, context, glob pat).
   my $class = shift;
   my $pat = pop;
   # Pass the context upwards, and also implicitly the superclass name.
   my $args = $class->SUPER::BUILDARGS(@_);
   return { %$args, pat => $pat };
}

sub _build_files {
   my ($self) = @_;
   # The given glob pattern is considered to be relative to the
   # current template.
   my $globarg = sprintf("%s/%s", $self->template->dir, $self->pat);
   return [ map { My::File->new(path => $_); } glob $globarg ];
}

no Moose;
__PACKAGE__->meta->make_immutable;
