package My::Json;
use Moose;
use JSON::XS;
use File::Slurp;
extends 'My::Plugin';

=encoding utf8

=head1 NAME

My::Json - a Template Toolkit plugin for reading a json file and
decoding its contents.

Depends on  B<My::Plugin>, B<JSON::XS> and B<File::Slurp>.

=head1 SYNOPSIS

You'd typically use this in a template to look up a value by a key,
the key usually being a file name or some string derived from a file
name.

For example, you have a file C<captions.json> containing

   { "fatcat": "This is an overfed cat",
     "kitten": "Cat in training"
   }

and in the same directory a template with

   [% USE caps = Json('captions.json') -%]
   <figure>
     <img src="fatcat.jpg">
     <figcaption>[% caps.value('fatcat') %]</figcaption>
   </figure>

then Template Toolkit will generate HTML with the caption "This is an
overfed cat".

In practice, you'd probably use this in conjuction with C<My::Glob> to
acquire a list of images and then, for each image, to look up its
caption.

=head1 CONSTRUCTION

Supply one argument, the name of the .json file, relative to the
directory of the template.  E.g. if the .json file is in the same
directory as the template then just use the plain file name, as in the
Synopsis:

   [% USE caps = Json('captions.json') -%]

=head1 METHODS

=over

=item B<value>

This plugin assumes that the json structure is a hash that decodes
into a perl hashref (because that's all I needed), so it provides a
method to look up a value by its key.  E.g.

   [% caps.value('fatcat') %]

=back

=head1 COPYING

©Copyright <webmaster@acrasis.net>. This is free software; you may
redistribute it and/or modify it under the same terms as Perl itself.

=cut

our $VERSION = 0.1;

has json_file => (is => 'ro', required => 1, isa => 'Str');

has json_ref => (
   is => 'ro', required => 0,
   isa => 'Ref', builder => '_build_json_ref', lazy => 1,
);

sub BUILDARGS {
   my $class = shift;
   my $json_file = pop;
   # Pass the context upwards, and also implicitly the superclass name.
   my $args = $class->SUPER::BUILDARGS(@_);
   return { %$args, json_file => $json_file };
}

sub _build_json_ref {
   my ($self) = @_;
   # The given name is considered to be relative to the current template.
   my $path = sprintf("%s/%s", $self->template->dir, $self->json_file);
   return decode_json(read_file($path));
}

sub value {
   my ($self, $key) = @_;
   return $self->json_ref->{$key};
}

no Moose;
__PACKAGE__->meta->make_immutable;
